# -*- coding: utf-8 -*-
from op5helpers import *
from nose.tools import nottest
from time import time, sleep

def request_json(resource=None, method=None, data=None, auth=None, endpoint='api/command'):
    uri = api_endpoint + endpoint
    if resource:
        uri = api_endpoint + endpoint + '/' + resource

    if not method:
        method = 'get'

    if not data:
        data = {}

    headers = {'content-type': 'application/json'}
    data = json.dumps(data)

    if not auth:
        auth = authed_user

    if method is 'get':
        r = requests.get(uri, auth=auth, data=data, headers=headers, verify=False)
    elif method is 'post':
        r = requests.post(uri, auth=auth, data=data, headers=headers, verify=False)
    try:
        _ = r.json()
    except ValueError:
        ok_(False, "Could not extract JSON (%d):\n%s" % (r.status_code, r.text))

    ok_('framework_error' not in r.text, "Response contains Kohana stack trace:\n"+r.text)

    return r

def test_anti_parameter_regression():
    cmds = {
        'SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME': {
            'host_name': 'string',
            'start_time': 'timestamp',
            'end_time': 'timestamp',
            'fixed': 'bool',
            'trigger_id': 'int',
            'duration': 'int',
            'comment': 'string'
        },
        'SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME': {
            'host_name': 'string',
            'start_time': 'timestamp',
            'end_time': 'timestamp',
            'fixed': 'bool',
            'trigger_id': 'int',
            'duration': 'int',
            'comment': 'string'
        },
        'SCHEDULE_SVC_DOWNTIME': {
            'host_name': 'string',
            'service_description': 'string',
            'start_time': 'timestamp',
            'end_time': 'timestamp',
            'fixed': 'bool',
            'trigger_id': 'int',
            'duration': 'int',
            'comment': 'string'
        },
        'SCHEDULE_HOST_DOWNTIME': {
            'host_name': 'string',
            'start_time': 'timestamp',
            'end_time': 'timestamp',
            'fixed': 'bool',
            'trigger_id': 'int',
            'duration': 'string',
            'comment': 'string'
        },
        'ACKNOWLEDGE_SVC_PROBLEM': {
            'host_name': 'string',
            'service_description': 'string',
            'sticky': 'int',
            'notify': 'bool',
            'persistent': 'bool',
            'comment': 'string'
        },
        'ACKNOWLEDGE_HOST_PROBLEM': {
            'host_name': 'string',
            'sticky': 'int',
            'notify': 'bool',
            'persistent': 'bool',
            'comment': 'string'
        },
        'SCHEDULE_HOST_CHECK': {
            'host_name': 'string',
            'check_time': 'timestamp'
        },
        'SCHEDULE_SVC_CHECK': {
            'host_name': 'string',
            'service_description': 'string',
            'check_time': 'timestamp'
        },
        'PROCESS_HOST_CHECK_RESULT': {
                'host_name': 'string',
                'status_code': 'int',
                'plugin_output': 'string'
        },
        'PROCESS_SERVICE_CHECK_RESULT': {
                'host_name': 'string',
                'service_description': 'string',
                'status_code': 'int',
                'plugin_output': 'string'
        }
    }

    def commandtest(cmd, params):
        r = request_json(cmd)
        eq_(200, r.status_code, "We wanted 200, got '%d' with the response body:\n%s" % (r.status_code, r.text))
        eq_(r.json()['parameters'], params)

    for cmd in cmds:
        yield commandtest, cmd, cmds[cmd]

    # ensure that we always have to keep this function in sync with every
    # exposed command through the API and thus always have the live version
    # handily tested. this really should be done for everything, everywhere,
    # and have breakages reported in the changelog
    r = request_json()
    eq_(200, r.status_code, "We wanted 200, got '%d' with the response body:\n%s" % (r.status_code, r.text))
    eq_(len(r.json()), len(cmds), "We should have %d commands, got %d in the response:\n%s" % (len(cmds), len(r.json()), r.text))
    for cmd in r.json():
        # all commands should have the two below defined keys
        eq_(sorted(r.json()[cmd].keys()), ['description', 'resource'])

def test_get_single_nonexistent():
    r = request_json('SEND_GRANDMA_TO_MARS_AND_PARTAY')
    eq_(r.status_code, 400, "Should have 400 as status code, got %d" % (r.status_code,))
    ok_('error' in r.json(), "Should have found 'error' in " + r.text)

def test_submit_nonexistent():
    r = request_json('SARASABA_SAYS_VOLDEMORT_IN_SNAKE_SHAPE')
    eq_(r.status_code, 400, "Should have 400 as status code, got %d" % (r.status_code,))
    ok_('error' in r.json(), "Should have found 'error' in " + r.text)

def test_authorized_ack_host_problem():
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'monitor',
        'sticky': 1,
        'notify': 1,
        'persistent': 1,
        'comment': "'ts an ack"
    })
    eq_(r.status_code, 200, "%d:\n%s" % (r.status_code, r.text))
    ok_('Successfully submitted ACKNOWLEDGE_HOST_PROBLEM' in r.text, "Bad message: " + r.text)

# might not work if non-default configuration. Requires local users and a guest-group with view-all permission
def test_unauthorized_ack_service_problem():
    crud = Crud('user', 'heltrandomguestaccount')

    data = {
        'username': 'heltrandomguestaccount',
        'password': 'guest',
        'groups': [
            'guest',
        ]}

    out = crud.post(data, cmp=False)

    r = request_json(resource='ACKNOWLEDGE_SVC_PROBLEM', method='post', auth=('heltrandomguestaccount', 'guest'), data={
        'host_name': 'monitor',
        'service_description': 'SSH Server',
        'sticky': 1,
        'notify': 1,
        'persistent': 1,
        'comment': 1,
        })
    crud.delete()
    crud.verify_gone()

    eq_(r.status_code, 403, "Wanted 403, got %d:\n%s" % (r.status_code, r.text))
    ok_('Forbidden' in r.text, "Bad message: " + r.text)

def test_too_many_input_parameters():
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'comment': 1,
        'fedora_hat': 'muy agradable',
        'host_name': 'monitor',
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 400, "Got %d:\n%s" % (r.status_code, r.text))
    ok_('does not take these parameters: fedora_hat' in r.text, "Bad message: " + r.text)

def test_too_few_input_parameters():
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'comment': 1,
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 400, "Got %d:\n%s" % (r.status_code, r.text))
    ok_('requires these parameters as well: host_name' in r.text, "Bad message: " + r.text)

#Skip because Nagios doesn't properly validate this yet
@nottest
def test_bad_input_values():
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'monitor',
        'comment': 1,
        'notify': 1,
        'sticky': 'your face',
        'persistent': 1,
    })
    eq_(r.status_code, 400, "Got %d:\n%s" % (r.status_code, r.text))
    eq_('Wrong type for argument', r.json()['error'], "Bad message: " + r.text)

def test_missing_object():
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'le host is missing - mayperhaps 12421h41',
        'comment': 1,
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 404, "Got %d:\n%s" % (r.status_code, r.text))
    eq_('Object not found', r.json()['error'], "Bad message: " + r.text)

def test_can_not_change_custom_vars_since_it_is_very_unsafe():
    for custom_var_command in [
        'CHANGE_CUSTOM_CONTACT_VAR',
        'CHANGE_CUSTOM_HOST_VAR',
        'CHANGE_CUSTOM_SVC_VAR']:
        r = request_json(resource=custom_var_command, method='post', data={
            'host_name': 'monitor',
            '_blaha': 'yuk',
        })
        eq_(r.status_code, 400, "Got %d:\n%s" % (r.status_code, r.text))
        ok_('Invalid command' in r.text, "Bad message: " + r.text)

def test_help_nonexisting_command():
    r = request_json(resource='shitbreak', method='get')
    eq_(r.status_code, 400, "Got %d:\n%s" % (r.status_code, r.text))
    eq_("The command 'shitbreak' doesn't exist", r.json()['error'], "Bad message: " + r.text)

def create_hosts():
    #Create down host
    crud = Crud('host', 'blackhawk')
    crud.post({
        'host_name': 'blackhawk',
        'address': 'not-a-proper-host.nowhere.org',
        'active_checks_enabled': True,
        'max_check_attempts': 16,
        'services': [{
            'service_description': 'my-service',
            'check_command': 'check-host-alive',
            'active_checks_enabled': True,
            'check_interval' : 1
            }]
        })

    crud = Crud('host', 'the-up-host')
    crud.post({
        'host_name': 'the-up-host',
        'address': 'localhost',
        'active_checks_enabled': True,
        'max_check_attempts': 16,
        })

    # child host: bald-eagle
    crud = Crud('host', 'bald-eagle')
    crud.post({
        'host_name': 'bald-eagle',
        'address': 'maybe-not-a-proper-host.nowhere.gov',
        'active_checks_enabled': True,
        'max_check_attempts': 20,
        'services': [{
            'service_description': 'my-service',
            'check_command': 'check-host-alive',
            'active_checks_enabled': True,
            'check_interval' : 1
            }],
        'parents': [
            'blackhawk'
            ]
        })

    # unrelated host that will be used for trigger id
    crud = Crud('host', 'swedish-chef')
    crud.post({
        'host_name': 'swedish-chef',
        'address': 'mammasimammasa.gov.edu.se',
        'active_checks_enabled': True,
        'services': [{
            'service_description': 'my-service',
            'check_command': 'check-host-alive',
            'active_checks_enabled': True,
            'check_interval' : 1
            }]
        })

    Change.post()

def delete_hosts():
    for host in ['blackhawk', 'the-up-host', 'bald-eagle', 'swedish-chef']:
        crud = Crud('host', host)
        crud.verify_gone()

    Change.post()

@with_setup(create_hosts, teardown=delete_hosts)
def test_detects_weird_chars_in_input():
    r = request_json(resource='PROCESS_HOST_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'status_code': 2,
        'plugin_output': 'Zomg!'
        })

    r = request_json(resource='host/the-up-host', endpoint='api/status')
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    num_comments = len(r.json()['comments'])
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'the-up-host',
        'comment': "You're a $bad mother~~, shaft\\!",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['result'], 'Successfully submitted ACKNOWLEDGE_HOST_PROBLEM')

    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'the-up-host',
        'comment': "Don't choke on this: ; fnutt",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))

    #Verify no comments created for host with no problems
    num_comments_after_test = len(dejson(request_json(resource='host/the-up-host', endpoint='api/status').text)['comments'])
    eq_(num_comments, num_comments_after_test, "Got %d, expected %d" % (num_comments_after_test, num_comments))

    host_status = dejson(request_json(resource='host/blackhawk', endpoint='api/status').text)
    ok_(0 != host_status['state'], "The tests requires blackhawk to not be up, but it was")

    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'blackhawk',
        'comment': "You're a $bad mother~~, shaft\\!",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))

    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'blackhawk',
        'comment': "Don't choke on this: ; fnutt",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))

    #Verify you can't escape the quoting
    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': 'blackhawk',
        'comment': "The man says \"Go take care of it\", I go take 'care' of't.",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))

    host_status = dejson(request_json(resource='host/blackhawk', endpoint='api/status').text)
    eq_(3, len(host_status['comments']), "Got %d comments:\n%s" % (len(host_status['comments']), host_status['comments']))
    eq_(host_status['comments_with_info'][1][2], "Don't choke on this: ; fnutt", "Tried to make sure semicolons survives PHP's shell mangling, got {0}\nFull response (look for comments_with_info):\n{1}".format(host_status['comments_with_info'][1][2], host_status))
    eq_(host_status['comments_with_info'][2][2], "The man says \"Go take care of it\", I go take 'care' of't.", "Tried to make sure quotes survives PHP's shell mangling, got {0}\nFull response (look for comments_with_info):\n{1}".format(host_status['comments_with_info'][2][2], host_status))
    eq_(1, host_status['acknowledged'], "Got %d" % host_status['acknowledged'])

@with_setup(create_hosts, delete_hosts)
def test_schedule_host_downtime():
    now = int(time())
    r = request_json(resource='SCHEDULE_HOST_DOWNTIME', method='post', data={
        'host_name' : 'blackhawk',
        'start_time' : now,
        'end_time' : now+5,
        'fixed' : 0, #flexible downtime
        'trigger_id' : 0,
        'duration' : 7200,
        'comment' : 'This host has 99 problems'
        })

    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['result'], 'Successfully submitted SCHEDULE_HOST_DOWNTIME')

    host_status = dejson(request_json(resource='host/blackhawk', endpoint='api/status').text)
    eq_(1, len(host_status['comments']), "Got %d:\n%s" % (len(host_status['comments']), host_status['comments']))

    now = int(time())
    r = request_json(resource='SCHEDULE_HOST_DOWNTIME', method='post', data={
        'host_name' : 'blackhawk',
        'start_time' : now ,
        'end_time' : now+7200,
        'fixed' : 1, #fixed downtime
        'trigger_id' : 0,
        'duration' : 0,
        'comment' : 'This host has 100 problems'
        })

    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['result'], 'Successfully submitted SCHEDULE_HOST_DOWNTIME')

    host_status = dejson(request_json(resource='host/blackhawk', endpoint='api/status').text)
    eq_(2, len(host_status['comments']), "Got %d:\n%s" % (len(host_status['comments']), host_status['comments']))
    eq_(2, len(host_status['downtimes']), "Got %d:\n%s" % (len(host_status['downtimes']), host_status['downtimes']))

@with_setup(create_hosts, delete_hosts)
def test_schedule_svc_downtime():
    now = int(time())
    r = request_json(resource='SCHEDULE_SVC_DOWNTIME', method='post', data={
        'host_name' : 'blackhawk',
        'service_description' : 'my-service',
        'start_time' : now,
        'end_time' : now+5,
        'fixed' : 0, #flexible downtime
        'trigger_id' : 0,
        'duration' : 7200,
        'comment' : 'This service is in flux!'
        })

    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['result'], 'Successfully submitted SCHEDULE_SVC_DOWNTIME')

    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(1, len(service_status['comments']), "Got %d:\n%s" % (len(service_status['comments']), service_status['comments']))
    eq_(1, len(service_status['downtimes']), "Got %d:\n%s" % (len(service_status['downtimes']), service_status['downtimes']))

@with_setup(create_hosts, delete_hosts)
def test_process_service_check_result():
    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    r = request_json(resource='PROCESS_SERVICE_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'service_description': 'my-service',
        'status_code': 2,
        'plugin_output': 'Zomg!'
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(2, service_status['state'], "Service status was not CRITICAL after a CRITICAL passive service check result\nWas %d" % service_status['state'])

    r = request_json(resource='PROCESS_SERVICE_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'service_description': 'my-service',
        'status_code': 0,
        'plugin_output': 'Zomg!'
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(0, service_status['state'], "Service status was not OK after a OK passive service check result\nWas %d" % service_status['state'])

@with_setup(create_hosts, delete_hosts)
def test_process_service_check_result_perfdata():
    perfdata = 'humidity=67;@55:63;@55'
    r = request_json(resource='PROCESS_SERVICE_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'service_description': 'my-service',
        'status_code': 0,
        'plugin_output': 'Zomg! | ' + perfdata
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(perfdata, service_status['perf_data'], "Service perfdata was incorrect after perfdata was passed\nWas %s, exprected %s" % (service_status['perf_data'], perfdata))

@with_setup(create_hosts, delete_hosts)
def test_process_host_check_result():
    host_name = 'blackhawk'
    host_status = dejson(request_json(resource='host/'+host_name, endpoint='api/status').text)
    r = request_json(resource='PROCESS_HOST_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'status_code': 2,
        'plugin_output': 'Zomg!'
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    host_status = dejson(request_json(resource='host/' + host_name, endpoint='api/status').text)
    eq_(2, host_status['state'], "Host status was not UNREACHABLE after a UNREACHABLE passive service check result\nWas %d" % host_status['state'])

    r = request_json(resource='PROCESS_HOST_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'status_code': 0,
        'plugin_output': 'Zomg!'
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    host_status = dejson(request_json(resource='host/' + host_name, endpoint='api/status').text)
    eq_(0, host_status['state'], "Host status was not UP after a UP passive service check result\nWas %d" % host_status['state'])

@with_setup(create_hosts, delete_hosts)
def test_acknowledge_svc_problem():
    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(0, service_status['acknowledged'], "Got %d" % service_status['acknowledged'])
    r = request_json(resource='PROCESS_SERVICE_CHECK_RESULT', method='post', data={
        'host_name': 'blackhawk',
        'service_description': 'my-service',
        'status_code': 2,
        'plugin_output': 'Oh noes!'
        })
    eq_(r.status_code, 200, "Got %d\n%s" % (r.status_code, r.text))
    r = request_json(resource='ACKNOWLEDGE_SVC_PROBLEM', method='post', data={
        'host_name': 'blackhawk',
        'service_description': 'my-service',
        'comment': "I acknowledge this problem, I'm totally working on it",
        'notify': 1,
        'sticky': 1,
        'persistent': 1,
    })
    eq_(r.status_code, 200, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['result'], 'Successfully submitted ACKNOWLEDGE_SVC_PROBLEM')

    service_status = dejson(request_json(resource='service/blackhawk;my-service', endpoint='api/status').text)
    eq_(1, service_status['acknowledged'], "Got %d" % service_status['acknowledged'])

@with_setup(create_hosts, delete_hosts)
def test_logged_in_user_propagates_author_field_implicitly():
    host_name = 'blackhawk'
    wanted_comment = 'Word to your mother'

    slept = 0
    host_status = dejson(request_json(resource='host/'+host_name, endpoint='api/status').text)
    eq_(0, host_status['acknowledged'], "Got %d" % host_status['acknowledged'])

    r = request_json(resource='PROCESS_HOST_CHECK_RESULT', method='post', data = {
        'host_name': 'blackhawk',
        'status_code': 2,
        'plugin_output': 'Zomg!'
        })

    eq_(r.status_code, 200, "%d:\n%s" % (r.status_code, r.text))

    r = request_json(resource='ACKNOWLEDGE_HOST_PROBLEM', method='post', data={
        'host_name': host_name,
        'sticky': 1,
        'notify': 1,
        'persistent': 1,
        'comment': wanted_comment
    })
    eq_(r.status_code, 200, "%d:\n%s" % (r.status_code, r.text))
    ok_('Successfully submitted ACKNOWLEDGE_HOST_PROBLEM' in r.text, "Bad message: " + r.text)

    r = requests.get(api_endpoint+'api/status/host/'+host_name, auth=authed_user, headers={'content-type': 'application/json'}, verify=False)
    eq_(r.status_code, 200, "%d:\n%s" % (r.status_code, r.text))
    host = r.json()

    ok_('comments_with_info' in host and len(host['comments_with_info']), "No comments found for host %s, there should be at least one:\n%s" % (host_name, str(host)))
    for comment in host['comments_with_info']:
        # also checks that the comments_with_info is an array of arrays, where the
        # inner array is [0] => id, [1] => author, [2] => comment
        if comment[2] != wanted_comment:
            continue
        eq_(comment[1], "monitor", "'%s' seems to have commented, but it should really have been 'monitor'" % comment[1])
        # we found the comment and got to test it, get outta hiier
        return
    raise Exception("Did not find the relevant '{0}' comment. All comments:\n{1}".format(wanted_comment, host['comments_with_info']))

@with_setup(create_hosts, delete_hosts)
def test_can_submit_crazy_values_for_start_and_end_time():
    r = request_json(resource='SCHEDULE_HOST_DOWNTIME', method='post', data={
        'host_name': 'blackhawk',
        'comment': 'dsafj',
        'start_time': 'timestamp',
        'end_time': 'timestamp',
        'fixed': 1,
        'trigger_id': 0,
        'duration': 0
    })
    eq_(400, r.status_code, "Nagios should validate these timestamps and thus mark this test as failed, which would be good. Got %d:\n%s" % (r.status_code, r.text))

@with_setup(create_hosts, delete_hosts)
def test_schedule_and_propagate_host_downtime():
    parent_name = 'blackhawk'
    child_name = 'bald-eagle'
    wanted_comment = 'seven, eight, gonna stay up late'
    now = int(time())

    r = request_json(resource='host/'+parent_name, endpoint='api/status')
    parent_status = dejson(r.text)
    try:
        eq_(0, parent_status['scheduled_downtime_depth'], "Got %d" % parent_status['scheduled_downtime_depth'])
    except KeyError:
        ok_(False, "Could not find a scheduled downtime depth in response (%d):\n%s" % (r.status_code, r.text))
    child_status = dejson(request_json(resource='host/'+child_name, endpoint='api/status').text)
    eq_(0, child_status['scheduled_downtime_depth'], "Child should not have had a scheduled downtime:\n%s" % child_status)

    r = request_json(resource='SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME', method='post', data={
        'host_name': parent_name,
        'comment': wanted_comment,
        'start_time': now,
        'end_time': now + 5,
        'fixed': 1,
        'trigger_id': 0,
        'duration': 0
    })
    eq_(200, r.status_code, "Could not schedule and propagate host downtime (%d):\n%s" % (r.status_code, r.text))

    child_status = dejson(request_json(resource='host/'+child_name, endpoint='api/status').text)
    eq_(1, child_status['scheduled_downtime_depth'], "Child should not have had a scheduled downtime:\n%s" % child_status)


@with_setup(create_hosts, delete_hosts)
def test_schedule_host_check():
    now = int(time())
    the_host = 'the-up-host'
    host_status = dejson(request_json(resource='host/'+the_host, endpoint='api/status').text)
    host_last_check = host_status['last_check']
    r = request_json(resource='SCHEDULE_HOST_CHECK', method='post', data={
        'host_name': the_host,
        'check_time': now
        })
    eq_(200, r.status_code, "Could not schedule host check (%d):\n%s" % (r.status_code, r.text))
    host_status = dejson(request_json(resource='host/'+the_host, endpoint='api/status').text)
    ok_(host_status['last_check'] >= host_last_check or host_status['next_check'] == now, "Host check not scheduled\n(last_check = %d, next_check = %d), expected %d for one of them" % (host_status['last_check'], host_status['next_check'], now))

@with_setup(create_hosts, delete_hosts)
def test_schedule_svc_check():
    the_host = 'blackhawk'
    the_service = 'my-service'
    now = int(time())
    service_status = dejson(request_json(resource='service/%s;%s' % (the_host, the_service), endpoint='api/status').text)
    service_last_check = service_status['last_check']
    r = request_json(resource='SCHEDULE_SVC_CHECK', method='post', data={
        'host_name': the_host,
        'service_description':  the_service,
        'check_time': now
        })
    eq_(200, r.status_code, "Could not schedule service check (%d):\n%s" % (r.status_code, r.text))
    service_status = dejson(request_json(resource='service/' + the_host + ';' + the_service, endpoint='api/status').text)
    ok_(service_status['last_check'] >= service_last_check or service_status['next_check'] == now, "Service check not scheduled\n(last_check = %d, next_check = %d), expected %d for one of them" % (service_status['last_check'], service_status['next_check'], now))

@with_setup(create_hosts, delete_hosts)
def test_schedule_and_propagate_triggered_host_downtime():
    host_that_triggers = 'swedish-chef'
    parent_name = 'blackhawk'
    child_name = 'bald-eagle'
    wanted_parent_comment = 'nine, ten, never sleep again'
    wanted_triggered_comment = "one, two, freddy is coming for you"
    now = int(time())

    parent = request_json(resource='host/'+parent_name, endpoint='api/status')
    parent_status = dejson(parent.text)
    eq_(200, parent.status_code, "Got %d:\n%s" % (parent.status_code, parent.text))
    eq_(0, parent_status['scheduled_downtime_depth'], "Got %d:\n%s" % (parent.status_code, parent.text))
    child = request_json(resource='host/'+child_name, endpoint='api/status')
    child_status = dejson(child.text)
    eq_(200, child.status_code, "Got %d:\n%s" % (child.status_code, child.text))
    eq_(0, child_status['scheduled_downtime_depth'], "Child should not have had a scheduled downtime (%d):\n%s" % (child.status_code, child.text))

    r = request_json(resource='SCHEDULE_HOST_DOWNTIME', method='post', data={
        'host_name': host_that_triggers,
        'comment': wanted_parent_comment,
        'start_time': now + 10,
        'end_time': now + 15,
        'fixed': 1,
        'trigger_id': 0,
        'duration': 0
    })
    eq_(200, r.status_code, "Could not schedule host downtime (%d):\n%s" % (r.status_code, r.text))
    parent_downtime = request_json(resource='host/'+host_that_triggers, endpoint='api/status').json()
    eq_(wanted_parent_comment, parent_downtime['downtimes_with_info'][0][2])
    ok_('downtimes_with_info' in parent_downtime and len(parent_downtime['downtimes_with_info']), "Could not trigger downtime for 'parent' (%d):\n%s" % (r.status_code, parent_downtime))
    trigger_id = parent_downtime['downtimes_with_info'][0][0]

    r = request_json(resource='SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME', method='post', data={
        'host_name': parent_name,
        'comment': wanted_triggered_comment,
        'start_time': now + 10,
        'end_time': now + 15,
        'fixed': 1,
        'trigger_id': trigger_id,
        'duration': 0
    })
    eq_(200, r.status_code, "Could not schedule and propagate host downtime (%d):\n%s" % (r.status_code, r.text))

    # we're making some assumptions here yo. *might* need some finlir depending
    # on the connection between your test runner and the Monitor SUT
    sleep(10)

    child_status = dejson(request_json(resource='host/'+child_name, endpoint='api/status').text)
    eq_(1, child_status['scheduled_downtime_depth'], "Child should not have had a scheduled downtime:\n%s" % child_status)
    eq_(wanted_triggered_comment, child_status['downtimes_with_info'][0][2])

# This let's you run
# nosetests -vv -a 'nagstamon' nose/*py
# and only have these tests run
# (google for "nose attributes" or such, the version of nosetests I'm running
# doesn't support the decorator syntax, thus this clunkiness)
test_schedule_svc_check.nagstamon = 1
test_schedule_host_check.nagstamon = 1
test_authorized_ack_host_problem.nagstamon = 1
test_unauthorized_ack_service_problem.nagstamon = 1
test_schedule_and_propagate_triggered_host_downtime.nagstamon = 1
test_schedule_svc_downtime.nagstamon = 1
