# -*- coding: utf-8 -*-

"""
Prepend the following string to all your tests

from op5helpers import *

Now build some nice tests!

"""

from nose.tools import *
import requests
import json
try: # the built-in one is too old for python 2.6
    import lxml.etree as etree
except ImportError:
    import xml.etree.ElementTree as etree
import urllib
import os
import sys
import subprocess
import time

# Avoids errors like these on SLES:
"""
/usr/lib64/python2.6/site-packages/requests/packages/urllib3/util/ssl_.py:100:
InsecurePlatformWarning: A true SSLContext object is not available. This
prevents urllib3 from configuring SSL appropriately and may cause certain SSL
connections to fail. For more information, see
https://urllib3.readthedocs.org/en/latest/security.html#insecureplatformwarning.

InsecurePlatformWarning
/usr/lib64/python2.6/site-packages/requests/packages/urllib3/connectionpool.py:789:
InsecureRequestWarning: Unverified HTTPS request is being made. Adding
certificate verification is strongly advised. See:
https://urllib3.readthedocs.org/en/latest/security.html InsecureRequestWarning)
"""
if hasattr(requests, 'packages'):
    requests.packages.urllib3.disable_warnings()

api_endpoint = os.environ.get("MONITOR_SERVER",'localhost')+'/'
if '://' not in api_endpoint:
    api_endpoint = 'https://'+api_endpoint
MERLIN_IMPORT = "/opt/monitor/op5/merlin/import"
authed_user = ('monitor', 'monitor')

try:
    assert_not_in
except NameError:
    def assert_in(member, container, msg=False):
        return assert_true(member in container, msg)
    def assert_not_in(member, container, msg=False):
        return assert_true(member not in container, msg)
    def assert_list_equal(lista, listb, msg=False):
        return eq_(lista, listb, msg)
    def assert_is_not_none(val, msg=False):
        return ok_(val is not None, msg)

def dejson(text):
    """ Get python structs (dicts/lists) from a json string """
    try:
        return json.loads(text)
    except ValueError:
        ok_(False, "Not a json object: " + text)

def dexml(text):
    """ Transform xml string to python's etree's form, fail if we fail to """
    try:
        tree = etree.fromstring(text)
    except etree.ParseError as e:
        ok_(False, "Not valid xml: " + text)
    return tree

def etree_nested_equal(exp, res):
    """ Deep comparison of xml """
    for expelem in exp:
        name = expelem.attrib.get('name', None)
        if name is None:
            was_found = False
            for reselem in res:
                try:
                    etree_nested_equal(expelem, reselem)
                    was_found = True
                except AssertionError:
                    pass
            ok_(was_found)
        else:
            reselem = res.find("param[@name='%s']" % name)
            assert_is_not_none(reselem, """Couldn't find element %s in output:

%s""" % (name, etree.tostring(res)))
            if not len(expelem):
                eq_(expelem.text, reselem.text)
            else:
                etree_nested_equal(expelem, reselem)

def nested_equal(exp, res):
    '''Validate that the keys in exp are also present in res.
    We intentionally ignore list ordering and element in res not in exp,
    as those are allowed to change nacoma-side.
    '''
    for attr in exp:
        if attr == "password":
            continue
        assert_in(attr, res, u"""
    Input element "%s" not found in output:

    %s
    """ % (attr, res))
        expval = exp[attr]
        resval = res[attr]
        if type(expval) == list:
            if expval and isinstance(expval[0], dict):
                eq_(len(expval), len(resval), u'%s should contain %i elements, has %i' % (attr, len(expval), len(resval)))
                for expelem in expval:
                    was_found = False
                    reasons = []
                    for reselem in resval:
                        try:
                            nested_equal(expelem, reselem)
                            was_found = True
                            break
                        except AssertionError as e:
                            reasons.append(unicode(e))
                    ok_(was_found, "Sub-element wasn't found. Reasons:\n%s\nExpected\n%s\nin\n%s" % ('\n'.join(reasons), expelem, resval))
                continue
            expval = set(expval)
        if type(resval) == list:
            if resval and isinstance(resval[0], dict):
                ok_(False, u"Unexpected dict for %s in output: %s" % (attr, resval))
            resval = set(resval)
        eq_(expval, resval, u'%s is different: expected "%s", was "%s"' % (attr, exp[attr], res[attr]))

def quote(stuff):
    """ For path that's prior to GET parameters """
    return urllib.quote(urllib.quote(stuff.encode('utf-8'), ''), '')

def singlequote(stuff):
    """ For GET parameters """
    return urllib.quote(stuff.encode('utf-8'), '')

def log(msg):
    """ Print a msg to stderr (will be visible in test log) """
    print >> sys.stderr, msg

class Crud(object):
    """ Shortcuts for crud-ing against the monitor api """

    type = False
    name = False
    data = False
    request_std_params = {'auth': authed_user, 'verify': False}

    @property
    def resource(self):
        return 'api/config/%s/%s' % (quote(self.type), quote(self.name))

    def __init__(self, type, name, clean=True, get_params=False):
        self.get_params = {'format': 'json'}
        if get_params:
            self.get_params.update(get_params)
        self.type = type
        self.name = name
        if clean:
            self.scrub_old()
        self.data = {}

    def scrub_old(self):
        r = requests.delete(api_endpoint + self.resource, params=self.get_params, **self.request_std_params)
        out = dejson(r.text)
        self.verify_gone()

    def verify_gone(self):
        r = requests.get(api_endpoint + 'api/config/%s' % self.type, params=self.get_params, **self.request_std_params)
        out = dejson(r.text)
        assert_not_in(self.name, out)

    def get(self, cmp=True, expected_code=200):
        url = api_endpoint + self.resource
        r = requests.get(url, params=self.get_params, **self.request_std_params)
        eq_(r.status_code, expected_code, "%s wasn't retrieved properly, wanted %d, got %d\nurl:\n%s\nbody:\n%s" % (self.name, expected_code, r.status_code, url, r.text))
        out = dejson(r.text)
        if cmp:
            nested_equal(self.data, out)
        return out

    def post(self, data, cmp=True, expected_code=201):
        self.data = data
        r = requests.post(api_endpoint + 'api/config/' + self.type,
            data=json.dumps(data),
            params=self.get_params,
            headers={'content-type': 'application/json'},
            **self.request_std_params)
        eq_(r.status_code, expected_code, "%s != %s. output: %s" % (r.status_code, expected_code, r.text))
        eq_(r.headers['Content-Type'], 'application/json; charset=utf-8',
            "Unexpected content type: " + r.headers['Content-Type'])
        out = dejson(r.text)
        if cmp:
            nested_equal(data, out)
        return out

    def put(self, data, cmp=True, expected_code=201):
        self.data = data
        r = requests.put(api_endpoint + self.resource,
            data=json.dumps(data),
            params=self.get_params,
            headers={'content-type': 'application/json'},
            **self.request_std_params)
        out = dejson(r.text)
        eq_(r.status_code, expected_code, "%s edit failed: %s" % (self.name, r.text))
        eq_(r.headers['Content-Type'], 'application/json; charset=utf-8',
            "Unexpected content type: " + r.headers['Content-Type'])
        if cmp:
            nested_equal(data, out)
        return out

    def patch(self, data, cmp=True, expected_code=200):
        self.data.update(data)
        r = requests.patch(api_endpoint + self.resource,
            data=json.dumps(data),
            params=self.get_params,
            headers={'content-type': 'application/json; charset=utf8'},
            **self.request_std_params)
        eq_(r.status_code, expected_code, "%s edit failed: %s" % (self.name, r.text))
        eq_(r.headers['Content-Type'], 'application/json; charset=utf-8',
            "Unexpected content type: " + r.headers['Content-Type'])
        out = dejson(r.text)
        if cmp:
            nested_equal(data, out)
        return out

    def delete(self, expected_code=200):
        r = requests.delete(api_endpoint + self.resource, params=self.get_params, **self.request_std_params)
        out = dejson(r.text)
        eq_(r.status_code, expected_code, "%s deletion failed: %s" % (self.resource, out))
        return out

def create_user(username, password, contact_for_hosts=None, contact_for_services=None):
    '''Helper to create both a user and a contact in one go'''
    Change.delete(False)
    crud = Crud('user', username)
    crud.post({
        'Username': username,
        'groups': [
            'limited_view'
        ],
        'modules': ['Default'],
        'Password': password}, cmp=False)
    crud = Crud('contact', username)
    crud.post({
        'contact_name': username,
        'alias': username})
    if contact_for_hosts:
        if isinstance(contact_for_hosts, str):
            contact_for_hosts = [contact_for_hosts]
        for host in contact_for_hosts:
            crud = Crud('host', host, clean=False)
            data = crud.get(cmp=False)
            data['contacts'].append(username)
            crud.put(data)
            crud.get()
    if contact_for_services:
        for service in contact_for_services:
            crud = Crud('service', service, clean=False)
            data = crud.get(cmp=False)
            data['contacts'].append(username)
            crud.put(data)
            crud.get()
    r = Change.post()
    eq_('Changes saved', dejson(r.text))

def delete_user(username):
    crud = Crud('user', username)
    crud = Crud('contact', username)
    crud = Crud('usergroup', 'user_' + username)
    r = Change.post()
    eq_('Changes saved', dejson(r.text))

def is_number(test):
    """ Check if string test is a number. Clunky, but seem to work. Stolen from
    http://stackoverflow.com/questions/354038/how-do-i-check-if-a-string-is-a-number-in-python """
    try:
        float(test)
        return True
    except ValueError:
        return False

class Change(object):
    """ Shortcut for handling persistance between Nacomas' db and Nagios' config files """
    request_std_params = {'auth': authed_user, 'verify': False}

    @staticmethod
    def uri():
        return api_endpoint+"api/config/change"

    @staticmethod
    def get(validate=True, error_message=None):
        r = requests.get(Change.uri(), **Change.request_std_params)
        if validate:
            eq_(r.status_code, 200, error_message or r.text)
        return r

    @staticmethod
    def delete(validate=True, error_message=None):
        r = requests.delete(Change.uri(), **Change.request_std_params)
        if validate:
            eq_(r.status_code, 200, error_message or r.text)
        return r

    @staticmethod
    def post(validate=True, error_message=None, sleep_seconds=5):
        r = requests.post(Change.uri(), **Change.request_std_params)
        if validate:
            eq_(r.status_code, 200, error_message or r.text)
        if sleep_seconds:
            time.sleep(sleep_seconds)
        return r

def import_log(name):
    """
    Decorator for report log tests

    import_log("simple") runs /opt/monitor/op5/merlin/import test/logs/simple.log

    It's also constructed as a decorator:

    @import_log("simple")
    def test_that_uses_data_from_simple_log():
        pass
    """
    args = [MERLIN_IMPORT, "--truncate-db"]
    logfile = os.path.abspath(os.path.dirname(__file__)) + "/../logs/%s.log" % (name)
    # when running tests, we shouldn't rely on global state.. retroactively
    # submitted downtimes might have been leftovers
    subprocess.check_output(['mysql', 'merlin', '-e', "truncate report_data"])
    subprocess.check_output(['mysql', 'merlin', '-e', "truncate report_data_extras"])
    if not os.path.isfile(logfile):
        raise Exception("Expecting to find the log file " + logfile)
    args.append(logfile)
    DEVNULL = open(os.devnull, 'wb')
    exit_code = subprocess.call(args, shell=False, stdout=DEVNULL)
    if exit_code:
        raise Exception("%d: Could not execute:\n%s" % (exit_code, ' '.join(args)))
    def mega_important_wrapper(f):
        return f
    return mega_important_wrapper

if "check_output" not in dir(subprocess):
    def f(*popenargs, **kwargs):
        """check_output() was introduced in 2.7, copy its (python) source from
        http://hg.python.org/cpython/file/d37f963394aa/Lib/subprocess.py#l544"""
        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, it will be overridden.')
        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise subprocess.CalledProcessError(retcode, cmd)
        return output
    subprocess.check_output = f
