<?php

require_once __DIR__.'/../../api/livestatus.php';
require_once __DIR__.'/../../api/lib/request.php';

class Api_Request_Mock extends Api_Request {
	function set_params(array $params) {
		$this->params = $params;
	}
}

/**
 * Unit tests for all examples provided in the "filter" paragraph of the
 * HTTP API's status help section.
 *
 * This predates the listview-query-filter language thingy by a year or two
 * but needs to be kept alive.
 *
 * Since we have a hard time fixturing an object with a soft state, we'll
 * perform unit tests instead of integration tests.
 */
class Api_Request_Backwards_Compatibility_Test extends PHPUnit_Framework_TestCase {
	function do_test(array $params, $expected_output) {
		$request = new Api_Request_Mock('not really needed, mind you');
		$request->set_params($params);
		$this->assertEquals($expected_output, Livestatus_Api::build_livestatus_query($request, 'host'));
	}

	function test_request_path_parsing_single_slashed ()  {
		$request = new Api_Request("/api/config/management_pack");
		$this->assertSame("config", $request->get_project_name());
		$this->assertSame(false, $request->is_help());
		$this->assertSame(array("management_pack"), $request->get_path());
	}

	function test_request_path_parsing_double_slashed ()  {
		$request = new Api_Request("//api/config/management_pack");
		$this->assertSame("config", $request->get_project_name());
		$this->assertSame(false, $request->is_help());
		$this->assertSame(array("management_pack"), $request->get_path());
	}

	function test_state()  {
		$this->do_test(array(
			'state' => 1
		), "GET hosts
Columns: name
Filter: state = 1
");
	}

	function test_states() {
		$this->do_test(array(
			'state' => array(1, 2)
		), "GET hosts
Columns: name
Filter: state = 1
Filter: state = 2
Or: 2
");
	}

	function test_state_and_hardness() {
		$this->do_test(array(
			'state' => 1,
			'hard' => 0
		), "GET hosts
Columns: name
Filter: state = 1
Filter: state_type = 0
");
	}

	function test_name() {
		$this->do_test(array(
			'name' => 'monitor'
		), "GET hosts
Columns: name
Filter: name ~ monitor
");
	}

	function test_name_state_hardness_format() {
		$this->do_test(array(
			'name' => 'linux',
			'state' => '0',
			'hard' => '1',
			'format' => 'xml'
		), "GET hosts
Columns: name
Filter: state = 0
Filter: state_type = 1
Filter: name ~ linux
");
	}

	function test_hostgroup() {
		$this->do_test(array(
			'hostgroup' => 'servers'
		), "GET hosts
Columns: name
Filter: groups >= servers
");
	}

}
